#include <Window.h>
#include <Message.h>
#include <View.h>
#include <StringView.h>
#include <TranslationUtils.h>
#include <Box.h>
#include <String.h>
#include <Bitmap.h>
#include <CheckBox.h>
#include "smsjoystick.h"
#include "JoyInputView.h"
#include <stdio.h>

struct input_lister
{
	const char *label;
	const char *text;
} joy_list[] = {
	{ "Button1:  ", "Enter The Key For button1" },
	{ "Button2:  ", "Enter The Key For button2" },
	{ "Start:  ", "Enter The Key For start" },
	{ 0, 0 }
};

JoyInputView::JoyInputView(BRect rect, BWindow *front, Joystick *joystick)
	: BView(rect, "", B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED),
	key(0),
	input_num(0),
	input(false),
	tile_bitmap(0),
	k(0)
{
	this->front = front;
	this->joystick = joystick;
	tile_bitmap = BTranslationUtils::GetBitmap("tile_bmap");
	
	box = new BBox(BRect(10, 10, 210, 210));
	box->SetLabel("Joystick:");
	AddChild(box);
	
	input_sviews[BUTTON1] = new BStringView(BRect(10, 20, 100, 35), "button1_input", "Button1:");
	box->AddChild(input_sviews[BUTTON1]);
	
	input_sviews[BUTTON2] = new BStringView(BRect(10, 40, 100, 55), "button2_input", "Button2:");
	box->AddChild(input_sviews[BUTTON2]);
	
	input_sviews[START] = new BStringView(BRect(10, 60, 100, 75), "start_input", "Start:");
	box->AddChild(input_sviews[START]);
	
	sview = new BStringView(BRect(10, 160, 160, 185), "label", joy_list[0].text);
	sview->SetFontSize(12);
	
	box->AddChild(sview);
	swapy = new BCheckBox(BRect(10, 80, 160, 40), "", "Swap V Axis", new BMessage('sway'));
	box->AddChild(swapy);
}

void JoyInputView::AttachedToWindow()
{
	swapy->SetTarget(this);
	SetViewBitmap(tile_bitmap);
}

void JoyInputView::MessageReceived(BMessage *m)
{
	switch(m->what)
	{
		case 'sway':
			p[0] = swapy->Value();
		break;
		default:
			BView::MessageReceived(m);
		break;
	}
}

void JoyInputView::Pulse()
{
	Window()->Lock();
	Draw(Bounds());
	Window()->Unlock();
}

void JoyInputView::Draw(BRect updateRect)
{
	sview->StrokeRect(BRect(0, 5, 150, 25));
	if(joystick)
	{
		joystick->Update();
		uint32 buttons_count = joystick->CountButtons();
		for(uint i = 0; i < buttons_count; i++)
		{
			if(joystick->Pressed(i))
			{
				key = i+1;
				k++;
				break;
			}
		}
	}
	if(IsInput() && k % 4 == 0) {
		if(key) {
			which[input_num] = key-1;
			ChangeLabel(input_num, key-1);
			input_num++;
			sview->SetText(joy_list[input_num].text);
			key = 0;
		}
		if(input_num >= MAX) { input_num = 0; sview->SetText(joy_list[input_num].text); }
		k = 0;
	}
}

void JoyInputView::ChangeLabel(int index, int ikey)
{
	BString string(joy_list[index].label);
	string << ikey;
	input_sviews[index]->SetText(string.String());
}

void JoyInputView::SetData(uint8 *which, bool *p)
{
	this->which = which;
	this->p = p;
	swapy->SetValue(p[0]);
	
	for(int i = 0; i < MAX; i++) {
		ChangeLabel(i, which[i]);
	}
}

JoyInputView::~JoyInputView()
{
	if(tile_bitmap) delete tile_bitmap;
}
