#include <stdio.h>
#include "smsinterface.h"
extern "C" {
#include "shared.h"
#include "loadzip.h"
};

/* Game file name */
char SmsInterface::game_name[0x100];
int SmsInterface::state_slot = 0;
FILE *SmsInterface::music_log = 0;
int SmsInterface::music_log_count = 0;

int SmsInterface::load_rom(char *filename)
{
	int size;
	strcpy(game_name, filename);
	
    if(strcmp(strrchr(filename, '.'), ".zip") == 0)
    {
        char name[0x100];

        cart.rom = loadzip(filename, name, &size);
        if(!cart.rom) return (0);
        printf("Loaded `%s', (%d bytes) from archive `%s'\n", name, size, filename);
        strcpy(game_name, name);

        /* Take care of image header, if present */
        if((size / 512) & 1)
        {
            size -= 512;
            memmove(cart.rom, cart.rom + 512, size);
        }

        cart.pages = (size / 0x4000);
    }
    else
    {
        FILE *fd = NULL;

        fd = fopen(filename, "rb");
        if(!fd) return (0);

        /* Seek to end of file, and get size */
        fseek(fd, 0, SEEK_END);
        size = ftell(fd);
        fseek(fd, 0, SEEK_SET);

        /* Don't load games smaller than 32k */
        if(size < 0x8000) return (0);

        /* Take care of image header, if present */
        if((size / 512) & 1)
        {
            size -= 512;
            fseek(fd, 512, SEEK_SET);
        }

        cart.pages = (size / 0x4000);
        cart.rom = (unsigned char *)malloc(size);
        if(!cart.rom) return (0);
        fread(cart.rom, size, 1, fd);

        fclose(fd);
    }

    /* Figure out game image type */
    if(strcmp(strrchr(game_name, '.'), ".gg") == 0)
        cart.type = TYPE_GG;
    else
        cart.type = TYPE_SMS;

    return (1);
}

/* Load SRAM data */
void SmsInterface::load_sram(void)
{
    char name[0x100];
    FILE *fd;
    strcpy(name, game_name);
    strcpy(strrchr(name, '.'), ".sav");
    fd = fopen(name, "rb");
    if(fd)
    {
        sms.save = 1;
        fread(sms.sram, 0x8000, 1, fd);
        fclose(fd);
    }
}

void system_load_sram(void)
{
    SmsInterface::load_sram();
}

void SmsInterface::log_handler(int data)
{
	if(music_log) fputc(data, music_log);
}

/* Save SRAM data */
void SmsInterface::save_sram(void)
{
    if(sms.save)
    {
        char name[0x100];
        FILE *fd = NULL;
        strcpy(name, game_name);
        strcpy(strrchr(name, '.'), ".sav");
        fd = fopen(name, "wb");
        if(fd)
        {
            fwrite(sms.sram, 0x8000, 1, fd);
            fclose(fd);
        }
    }
}

/* Load system state */
int SmsInterface::load_state(bool ok, int number)
{
	if(!ok) return(0);
    char name[0x100];
    FILE *fd = NULL;
    strcpy(name, game_name);
    sprintf(strrchr(name, '.'), ".st%d", number);//state_slot);
    fd = fopen(name, "rb");
    if(!fd) return (0);
    system_load_state(fd);
    fclose(fd);
    return (1);
}

/* Save system state */
int SmsInterface::save_state(bool ok, int number)
{
	if(!ok) return(0);
    char name[0x100];
    FILE *fd = NULL;
    strcpy(name, game_name);
    sprintf(strrchr(name, '.'), ".st%d", number);//state_slot);
    fd = fopen(name, "wb");
    if(!fd) return (0);
    system_save_state(fd);
    fclose(fd);
    return(1);
}

void SmsInterface::stop_log(void)
{
	if(music_log)
	{
		snd.log = 0;
		fclose(music_log);
		music_log = NULL;
	}
}

void SmsInterface::log_audio(void)
{
	if(music_log)
	{
		snd.log = 0;
		fclose(music_log);
		music_log = NULL;
	}
	else
	{
		char name[0x100];
		char ext[0x100];
		sprintf(ext, "%02d.ssl", music_log_count);
		strcpy(name, game_name);
		strcpy(strrchr(name, '.'), ext);
		snd.log = 1;
		music_log_count += 1;
		music_log = fopen(name, "wb");
	}
}