#include "smssoundplayer.h"

SmsSoundPlayer::SmsSoundPlayer(const media_raw_audio_format * format, const char * name = NULL)
	: BSoundPlayer(format, name),
	sub_buffer(0)
{
	SetBufferPlayer(callback);
	SetCookie(this);
}

void SmsSoundPlayer::InitAll(int sub_buffer_size)
{
	sub_buffer = new uint8[2*sub_buffer_size];
	memset(sub_buffer, 0, sub_buffer_size);
	write_buffer = 0;
	read_buffer = 0;
	audio_len = sub_buffer_size;
	this->Start();
}

void SmsSoundPlayer::WriteBufferSync(short *data, int len)
{
	int start, end;
	while((write_buffer - read_buffer) > (audio_len - len)) { snooze(1000); }
	
	locker.Lock();
	
	start = write_buffer % audio_len;
	end = (start + len) % audio_len;
	
	if(end < start) {
		memcpy(sub_buffer + start, data, audio_len - start);
		memcpy(sub_buffer, data + audio_len - start, end);
	}
	else
		memcpy(sub_buffer + start, data, len);
	
	write_buffer += len;
	
	locker.Unlock();
}

void SmsSoundPlayer::callback(void *cookie, void *buffer, size_t len, const media_raw_audio_format &format)
{
	SmsSoundPlayer *s = (SmsSoundPlayer *)cookie;
	s->locker.Lock();
	int start, end;
	short *stream = (short *)buffer;
	if((s->write_buffer - s->read_buffer) < (int)len) {
		s->read_buffer += len;
		s->locker.Unlock();
		return;
	}
	
	start = s->read_buffer % s->audio_len;
	end = (start + len) % s->audio_len;
	
	if(end < start) {
		memcpy(stream, s->sub_buffer + start, s->audio_len - start);
		memcpy(stream + s->audio_len - start, s->sub_buffer, end);
	}
	else
		memcpy(stream, s->sub_buffer + start, len);

	s->read_buffer += len;
	s->locker.Unlock();
}

void SmsSoundPlayer::ResetAll(bool value)
{
	write_buffer = 0;
	read_buffer = 0;
	if(locker.IsLocked()) locker.Unlock();
	memset(sub_buffer, 0, audio_len);
	this->SetHasData(value);
}

void SmsSoundPlayer::Lock()
{
	locker.Lock();
}

void SmsSoundPlayer::Unlock()
{
	locker.Unlock();
}

SmsSoundPlayer::~SmsSoundPlayer()
{
	delete [] sub_buffer;
}