#include "smswindow.h"
#include "smskeyinputwindow.h"
#include "smsjoyinputwindow.h"
#include "system_beos_constants.h"
#include "system_menu_bar.h"
#include "2xsaidefines.h"
#include "smsinterface.h"
#include "smssoundplayer.h"
#include "smsjoystick.h"
#include "smsrefreshwindow.h"
extern "C" {
#include "osd_beos.h"
};

uint8 snes9x_clear_change_log = 0; // this is used inside 2xsai asm

SmsWindow::SmsWindow(BRect rect)
	: BDirectWindow(rect, "SmsPlus", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE),
	sms_key0_input_window(0),
	sms_key1_input_window(0),
	sms_joy0_input_window(0),
	sms_joy1_input_window(0),
	sms_refresh_window(0),
	sms_bmp(0),
	sms_back_bmp(0),
	sms_filepanel(0),
	sms_running(false),
	sms_quitting(true),
	frame_count(0),
	frames_rendered(0),
	frame_skip(1),
	skip(0),
	m_cliplist(NULL),
	m_connected(false),
	needs_clear(false),
	joystick0(0),
	joystick1(0),
	mode_is_valid(false)
{
	GetSettings(settings);
	
	sms_bmp = new BBitmap(BRect(0, 0, 255, 191), B_RGB16);
	sms_back_bmp = new BBitmap(BRect(0, 0, 511, 383), B_RGB16);
	memset(sms_bmp->Bits(), 0, sms_bmp->BitsLength());
	memset(key, 0, sizeof(key));
	
	rect.OffsetTo(B_ORIGIN);
	InitMenuBar();
	
	rect.top = (float)_menubar_height;
	sms_view = new BView(rect, "", B_FOLLOW_ALL, B_WILL_DRAW);
	sms_view->SetViewColor(0, 0, 0);
	AddChild(sms_view);
	
	m_locker = new BLocker();
	sms_filepanel = new BFilePanel(B_OPEN_PANEL, new BMessenger(this), NULL, B_FILE_NODE, false);
	AddShortcut('F', B_COMMAND_KEY, new BMessage('full'));
	
	sms_lock_sem = create_sem(0, "sms_lock_sem");
	
	sms_draw_id = spawn_thread(sms_drawing_thread, "sms_drawing_thread", B_NORMAL_PRIORITY, this);
	resume_thread(sms_draw_id);
	
	sms_id = spawn_thread(sms_thread, "sms_thread", B_URGENT_DISPLAY_PRIORITY, this);
	resume_thread(sms_id);
	
	settings.uisettings.mode > 0 ? ResizeTo(511, 383) : ResizeTo(255, 191);
	display_mode *m_mode;
	uint32 m_nummodes;
	
	if(BScreen().GetModeList(&m_mode, &m_nummodes) == B_NO_ERROR)
	{
		for (unsigned int i = 0; i < m_nummodes; i++)
		{
			if((m_mode[i].virtual_width == 640) && (m_mode[i].space == B_RGB16))
			{
				mode_is_valid = true;
				new_mode = m_mode[i];
				break;
			}
		}
	}
	BScreen().GetMode(&prev_mode);
}

void SmsWindow::InitMenuBar()
{
	g_menu = new GameMenuBar(BRect(0, 0, 0, 0), "menu_bar");
	AddChild(g_menu);
	_menubar_height = (int32)g_menu->Bounds().Height();
}

void SmsWindow::ResizeTo(float width, float height)
{
	height += _menubar_height+1;
	BDirectWindow::ResizeTo(width, height);
}

void SmsWindow::WindowActivated(bool active)
{
	if(active && sms_running) needs_clear = true;
	BDirectWindow::WindowActivated(active);
}

int32 SmsWindow::sms_input_thread(void *o)
{
	SmsWindow *w = (SmsWindow *)o;
	return w->sms_input_thread();
}

int32 SmsWindow::sms_input_thread() // Be's multipede
{
	bigtime_t nextTime, snoozeTime, baseTime;
	baseTime = system_time();
	while(1) {
		nextTime = baseTime + 66666;
		snoozeTime = (nextTime - system_time());
		
		while (snoozeTime >= 1.0) {
			snooze(snoozeTime);
			snoozeTime = nextTime - system_time();
		}
		update_input();
		baseTime = nextTime;
	}
}

int32 SmsWindow::sms_thread(void *o)
{
	SmsWindow *w = (SmsWindow *)o;
	return w->sms_thread();
}

int32 SmsWindow::sms_thread()
{
	ui_init();
	snooze(32000);
	input_init();
	Init_2xSaI(565);
	sms_quitting = false;
	
	while(!sms_quitting) {
		if(sms_quitting) break;
		if(!sms_running) {
			snooze(16000);
		}
		else
			sms_emulate();
	}
	return(0);
}

void SmsWindow::update_video()
{
	if(m_format != B_RGB16) { DrawMessage((unsigned char *)"16 bits only", 0, 0); return; }
	
	switch(settings.uisettings.mode) {
		case 0:
			#include "1x_normal.h"
		break;
		case 1:
			#include "2x_double_mode.h"
		break;
		case 2:
			#include "2x_doubled_scanlines.h"
		break;
		case 3:
			#include "2x_lcd_mode.h"
		break;
	}
}

void SmsWindow::update_videofull()
{
	switch(settings.uisettings.mode) {
		case 0:
			#include "1x_normalfull.h"
		break;
		case 1:
			#include "2x_double_modefull.h"
		break;
		case 2:
			#include "2x_doubled_scanlinesfull.h"
		break;
		case 3:
			#include "2x_lcd_modefull.h"
		break;
		case 4:
			#include "2xsaifull.h"
		break;
		case 5:
			#include "super2xsaifull.h"
		break;
		case 6:
			#include "supereaglefull.h"
		break;
	}
}

void SmsWindow::ClearScreen()
{
	uint32 *start = (uint32 *)sms_back_bmp->Bits();
	uint32 *end = (uint32 *)sms_back_bmp->Bits() + (sms_back_bmp->BitsLength() >> 2);
	while(start < end)
		*start++ ^= ~0;
}

int32 SmsWindow::sms_drawing_thread(void *o)
{
	SmsWindow *w = (SmsWindow *)o;
	return w->sms_drawing_thread();
}

int32 SmsWindow::sms_drawing_thread()
{
	while(acquire_sem(sms_lock_sem) == B_OK) {
		if(needs_clear) { needs_clear = false; ClearScreen(); if(IsFullScreen()) memset(m_bits, 0, ((int32)(Bounds().bottom+1)*m_rowbytes)); }
		IsFullScreen() ? update_videofull() : update_video();
	}
	return(0);
}

void SmsWindow::StartTime(bigtime_t &starttime)
{
	CheckMenuItem(ID_LOG_AUDIO, false);
	SmsInterface::stop_log();
	starttime = system_time();
	frame_count = 0;
	frames_rendered = 0;
	frame_skip = 1;
	skip = 0;
}

void SmsWindow::sms_emulate()
{
	static bigtime_t prev_tick = 0;
	static const bigtime_t frame_snooze = 1000000 / 60;
	
	/* Bump frame count */
	frame_count += 1;
	frames_rendered += 1;
	
	skip = (frame_count % frame_skip == 0) ? 0 : 1;
	sms_frame(skip);
	
	/* Update the display */
	if(skip == 0)
	{
		if(settings.uisettings.mute == false) update_audio();
		release_sem_etc(sms_lock_sem, 1, B_DO_NOT_RESCHEDULE);
	}
	
	if(settings.uisettings.mute) {
		bigtime_t snooze_time = frame_snooze - (system_time()-prev_tick);
		if (snooze_time > 0) snooze(snooze_time);
		prev_tick = system_time();
	}
}

void SmsWindow::init_machine()
{
	/* Set up bitmap structure */
	memset(&bitmap, 0, sizeof(t_bitmap));
	bitmap.width  = sms_bmp->Bounds().IntegerWidth() + 1;
	bitmap.height = sms_bmp->Bounds().IntegerHeight() + 1;
	bitmap.depth  = 16;
	bitmap.pitch  = sms_bmp->BytesPerRow();
	bitmap.data   = (unsigned char *)sms_bmp->Bits();
}

bool SmsWindow::LoadAndRun(const entry_ref &ref)
{
	char temp[256];
	BPath path;
	BEntry entry(&ref, true);
	sms_running = false;
	
	if(sms_player) sms_player->ResetAll(false);
	
	SmsInterface::save_sram();
	StartTime(start);
	
	if(entry.GetPath(&path) != B_OK) return false;
	
	if(SmsInterface::load_rom((char *)path.Path()) == 0) {
		sprintf(temp, "Error `%.255s'. is not a valid Sms Game\n", (char *)path.Path());
		(new BAlert("sms_plus", temp, "Ok", NULL, NULL, B_WIDTH_FROM_LABEL))->Go(NULL);
		return false;
	}
	
	option.sound        =   1;
    option.sndcard      =   2;
    option.sndrate      =   44100;
    option.fm_enable    =   1;
	/* Set up video, audio, input, etc. */
    init_machine();
    
	/* Initialize the virtual console emulation */
    system_init(option.sound ? option.sndrate : 0);
    sms.country = option.country;
    sms.use_fm = option.fm_enable;
    snd.callback = SmsInterface::log_handler;
    
	/* Load SRAM from disk if possible */
	SmsInterface::load_sram();
	if(sms_player) sms_player->ResetAll(true);
	sms_running = true;
	return true;
}

void SmsWindow::update_input()
{
	/* Clear all button states */
    memset(&input, 0, sizeof(t_input));
    
    if(joystick0)
    {
    	joystick0->Update();
    	if(joystick0->Up()) input.pad[0] |= INPUT_UP;
    	if(joystick0->Down()) input.pad[0] |= INPUT_DOWN;
    	if(joystick0->Left()) input.pad[0] |= INPUT_LEFT;
    	if(joystick0->Right()) input.pad[0] |= INPUT_RIGHT;
    	
    	if(joystick0->Pressed(settings.joypad0.jbutton2)) input.pad[0] |= INPUT_BUTTON2;
    	if(joystick0->Pressed(settings.joypad0.jbutton1)) input.pad[0] |= INPUT_BUTTON1;
		if(joystick0->Pressed(settings.joypad0.jstart))  input.system |= (IS_GG) ? INPUT_START : INPUT_PAUSE;
    }
    
    if(joystick1)
    {
    	joystick1->Update();
    	if(joystick1->Up()) input.pad[1] |= INPUT_UP;
    	if(joystick1->Down()) input.pad[1] |= INPUT_DOWN;
    	if(joystick1->Left()) input.pad[1] |= INPUT_LEFT;
    	if(joystick1->Right()) input.pad[1] |= INPUT_RIGHT;
    	
    	if(joystick1->Pressed(settings.joypad1.jbutton2)) input.pad[1] |= INPUT_BUTTON2;
    	if(joystick1->Pressed(settings.joypad1.jbutton1)) input.pad[1] |= INPUT_BUTTON1;
		if(joystick1->Pressed(settings.joypad1.jstart))  input.system |= (IS_GG) ? INPUT_START : INPUT_PAUSE;
    }
    
    /* System buttons */
    if(key[settings.keypad0.kstart]) input.system |= (IS_GG) ? INPUT_START : INPUT_PAUSE;
    
    /* Player 1 buttons */
    if(key[settings.keypad0.kup])         input.pad[0] |= INPUT_UP;
    else
    if(key[settings.keypad0.kdown])       input.pad[0] |= INPUT_DOWN;
    if(key[settings.keypad0.kleft])       input.pad[0] |= INPUT_LEFT;
    else
    if(key[settings.keypad0.kright])      input.pad[0] |= INPUT_RIGHT;
    if(key[settings.keypad0.kbutton2])   input.pad[0] |= INPUT_BUTTON2;
    if(key[settings.keypad0.kbutton1])        input.pad[0] |= INPUT_BUTTON1;
    
    /* Player 2 buttons */
    if(key[settings.keypad1.kup])         input.pad[1] |= INPUT_UP;
    else
    if(key[settings.keypad1.kdown])       input.pad[1] |= INPUT_DOWN;
    if(key[settings.keypad1.kleft])       input.pad[1] |= INPUT_LEFT;
    else
    if(key[settings.keypad1.kright])      input.pad[1] |= INPUT_RIGHT;
    if(key[settings.keypad1.kbutton2])   input.pad[1] |= INPUT_BUTTON2;
    if(key[settings.keypad1.kbutton1])        input.pad[1] |= INPUT_BUTTON1;
}

void SmsWindow::MessageReceived(BMessage *m)
{
	int32 index;
	switch(m->what) {
		case B_KEY_DOWN:
		case B_UNMAPPED_KEY_DOWN:
			if(m->FindInt32("key", &index) == B_NO_ERROR)
				key[index] = 1;
		break;
		case B_KEY_UP:
		case B_UNMAPPED_KEY_UP:
			if(m->FindInt32("key", &index) == B_NO_ERROR)
				key[index] = 0;
		break;
		case 'open':
			sms_filepanel->Show();
		break;
		case 'full':
			if(!mode_is_valid) break;
			SetFullScreen(!IsFullScreen());
			if(!IsFullScreen()) {
				AddChild(g_menu);
				sms_view->MoveTo(0, _menubar_height);
				BScreen().SetMode(&prev_mode);
			}
			if(IsFullScreen()) {
				RemoveChild(g_menu);
				sms_view->MoveTo(B_ORIGIN);
				sms_view->ResizeBy(0, _menubar_height);
				BScreen().SetMode(&new_mode);
			}
			needs_clear = true;
		break;
		case B_SIMPLE_DATA:
		case B_REFS_RECEIVED: {
			entry_ref ref;
			if(m->FindRef("refs", &ref) == B_NO_ERROR) {
				LoadAndRun(ref);
			}
		} break;
		case ID_GAME_EXIT:
			PostMessage(B_QUIT_REQUESTED);
		break;
		case ID_OPTIONS_MUTE:
			settings.uisettings.mute = !settings.uisettings.mute;
			CheckMenuItem(ID_OPTIONS_MUTE, settings.uisettings.mute);
			if(sms_player && sms_running) sms_player->ResetAll(settings.uisettings.mute ? false : true);
		break;
		case ID_LAUNCH_KEYBOARD0:
			if(sms_key0_input_window != 0) break;
			sms_key0_input_window = new SmsKeyInputWindow(BRect(200, 200, 420, 420), "SmsPlus - Keyboard 0",
				(uint8 *)&settings.keypad0, this, 'key0');
			sms_key0_input_window->Show();
		break;
		case ID_LAUNCH_KEYBOARD1:
			if(sms_key1_input_window != 0) break;
			sms_key1_input_window = new SmsKeyInputWindow(BRect(200, 200, 420, 420), "SmsPlus - Keyboard 1",
				(uint8 *)&settings.keypad1, this, 'key1');
			sms_key1_input_window->Show();
		break;
		case ID_LAUNCH_JOYSTICK0:
			if(sms_joy0_input_window != 0) break;
			sms_joy0_input_window = new SmsJoyInputWindow(BRect(200, 200, 420, 420), "SmsPlus - Joystick 0",
				(uint8 *)&settings.joypad0, this, 'joy0', joystick0, (bool *)&settings.joypad0.swapy);
			sms_joy0_input_window->Show();
		break;
		case ID_LAUNCH_JOYSTICK1:
			if(sms_joy1_input_window != 0) break;
			sms_joy1_input_window = new SmsJoyInputWindow(BRect(200, 200, 420, 420), "SmsPlus - Joystick 1",
				(uint8 *)&settings.joypad1, this, 'joy1', joystick1, (bool *)&settings.joypad1.swapy);
			sms_joy1_input_window->Show();
		break;
		case ID_LAUNCH_REFRESH:
			if(sms_refresh_window != 0) break;
			if(!mode_is_valid) break;
			sms_refresh_window = new SmsRefreshWindow(BRect(200, 200, 380, 300), this, 'redl');
			sms_refresh_window->Show();
		break;
		case 'key0':
			sms_key0_input_window = 0;
		break;
		case 'key1':
			sms_key1_input_window = 0;
		break;
		case 'joy0':
			sms_joy0_input_window = 0;
			if(joystick0) joystick0->SwapVerticalAxes(settings.joypad0.swapy);
		break;
		case 'joy1':
			sms_joy1_input_window = 0;
			if(joystick1) joystick1->SwapVerticalAxes(settings.joypad1.swapy);
		break;
		case 'redl':
			sms_refresh_window = 0;
		break;
		case ID_OPTIONS_SPECIAL_MODE0:
			CheckMenuItem(ID_OPTIONS_SPECIAL_MODE0, true);
			SetGraphicsMode(0);
		break;
		case ID_OPTIONS_SPECIAL_MODE1:
			CheckMenuItem(ID_OPTIONS_SPECIAL_MODE1, true);
			SetGraphicsMode(1);
		break;
		case ID_OPTIONS_SPECIAL_MODE2:
			CheckMenuItem(ID_OPTIONS_SPECIAL_MODE2, true);
			SetGraphicsMode(2);
		break;
		case ID_OPTIONS_SPECIAL_MODE3:
			CheckMenuItem(ID_OPTIONS_SPECIAL_MODE3, true);
			SetGraphicsMode(3);
		break;
		case ID_OPTIONS_SPECIAL_MODE4:
			CheckMenuItem(ID_OPTIONS_SPECIAL_MODE4, true);
			SetGraphicsMode(4);
		break;
		case ID_OPTIONS_SPECIAL_MODE5:
			CheckMenuItem(ID_OPTIONS_SPECIAL_MODE5, true);
			SetGraphicsMode(5);
		break;
		case ID_OPTIONS_SPECIAL_MODE6:
			CheckMenuItem(ID_OPTIONS_SPECIAL_MODE6, true);
			SetGraphicsMode(6);
		break;
		case ID_LOG_AUDIO: {
			if(!sms_running) break;
			BMenuItem *item;
			m->FindPointer("source", (void **)&item);
			item->SetMarked(!item->IsMarked());
			SmsInterface::log_audio();
		}
		break;
		case ID_SAVE_STATE: {
			int32 slot;
			m->FindInt32("save_slot", &slot);
			SmsInterface::save_state(sms_running, slot);
		}
		break;
		case ID_LOAD_STATE: {
			int32 slot;
			m->FindInt32("load_slot", &slot);
			SmsInterface::load_state(sms_running, slot);
		}
		break;
		case ID_GAME_SOFT_RESET:
			input.system |= (IS_GG) ? INPUT_HARD_RESET : INPUT_SOFT_RESET;
		break;
		case ID_GAME_HARD_RESET:
			input.system |= INPUT_HARD_RESET;
		break;
		case ID_HELP_ABOUTSMSPLUS:
			DoAlert();
		break;
		case ID_REFRESH: {
			display_mode *mode;
			if(m->FindPointer("refr_rate", (void **)&mode) == B_NO_ERROR) {
				new_mode = *mode;
			}
		}	
		break;
		default:
			BDirectWindow::MessageReceived(m);
		break;
	}
}

void SmsWindow::DoAlert()
{
	BString text;
	text << "SmsPlus " << "0.9.4a.1" << "\n\n" << "Dos version and Emulation Core: \nby Charles Mac Donald\n" << "\nBeOS version: \nby Caz (Carwyn Jones)\n"
	<< "\nBeOS testing and thanks to: \nChristian Corona\nGoodOldGames\nJack Burton\n"
	<< "\n" << "This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation;"
		<< "\n\neither version 2 of the License, or (at your option) any later version. See also the license.txt file for additional informations.";
	(new BAlert("neopop", text.String(), "Ok", NULL, NULL, B_WIDTH_FROM_LABEL))->Go(NULL);
}

void SmsWindow::EnableMenuItem(const uint32 msgwhat, bool enable)
{
	BMenuItem *item = g_menu->FindItem(msgwhat);
	if(item != NULL) item->SetEnabled(enable);
}

void SmsWindow::CheckMenuItem(const uint32 msg, bool check)
{
	BMenuItem *item = g_menu->FindItem(msg);
	if(item != NULL) item->SetMarked(check);
}

void SmsWindow::SetGraphicsMode(int g_mode)
{
	settings.uisettings.mode = g_mode;
	settings.uisettings.mode > 0 ? ResizeTo(511, 383) : ResizeTo(255, 191);
	if(settings.uisettings.mode > 0 && m_connected && m_format == B_RGB16) DirectClear();
}

void SmsWindow::DirectClear()
{
	#include "system_clear_direct.h"
}

void SmsWindow::DirectConnected(direct_buffer_info *info)
{
	uint32 j;
	
	switch(info->buffer_state & B_DIRECT_MODE_MASK) {
		case B_DIRECT_START:
		case B_DIRECT_MODIFY:
			m_locker->Lock();
			if (m_cliplist) {
				free(m_cliplist);
				m_cliplist = NULL;
			}
			m_numcliprects = info->clip_list_count; 
			m_cliplist = (clipping_rect *) 
				malloc(m_numcliprects*sizeof(clipping_rect));
			if (m_cliplist) {
				memcpy(m_cliplist, info->clip_list,
					m_numcliprects*sizeof(clipping_rect));
				
				m_rowbytes = info->bytes_per_row;
				m_format = info->pixel_format;
				m_bitsperpixel = info->bits_per_pixel>>3;
				m_bounds = info->window_bounds;
				
				if(!IsFullScreen()) {
					m_bounds.top += _menubar_height + 1;
					m_bits = (uint8 *)info->bits + (m_bounds.top * m_rowbytes);
					
					for(j = 0; j < m_numcliprects; j++)
						if (m_cliplist[j].top <= (long)(info->window_bounds.top + _menubar_height))
							m_cliplist[j].top = info->window_bounds.top + _menubar_height + 1;
				} else {
					switch(settings.uisettings.mode) {
					case 0:
					for(j = 0; j < (uint32)m_numcliprects; j++)
					{
						m_cliplist[j].left = info->window_bounds.left + ((info->window_bounds.right)/2) - ((int32)sms_back_bmp->Bounds().right/4);
						m_cliplist[j].top = info->window_bounds.top + ((info->window_bounds.bottom)/2) - ((int32)sms_back_bmp->Bounds().bottom/4);
						m_cliplist[j].bottom = ((m_cliplist[j].top)) + ((int32)(sms_back_bmp->Bounds().bottom)/2);
						m_cliplist[j].right = (m_cliplist[j].left) + ((int32)(sms_back_bmp->Bounds().right)/2);
					}
					break;
					case 1 ... 7:
					for(j = 0; j < (uint32)m_numcliprects; j++)
					{
						m_cliplist[j].left = info->window_bounds.left + ((info->window_bounds.right)/2) - ((int32)sms_back_bmp->Bounds().right/2);
						m_cliplist[j].top = info->window_bounds.top + ((info->window_bounds.bottom)/2) - ((int32)sms_back_bmp->Bounds().bottom/2);
						m_cliplist[j].bottom = ((m_cliplist[j].top)) + ((int32)(sms_back_bmp->Bounds().bottom));
						m_cliplist[j].right = (m_cliplist[j].left) + ((int32)(sms_back_bmp->Bounds().right));
					}
					break;
					}
					m_bits = (uint8 *)info->bits;
				}
				m_connected = true;
			}
			m_locker->Unlock();
		break;
		case B_DIRECT_STOP:
			m_locker->Lock();
			m_connected = false;
			m_locker->Unlock();
		break;
	}
}

bool SmsWindow::sound_init()
{
	media_raw_audio_format format = {
		44100.0,
		2,
		media_raw_audio_format::B_AUDIO_SHORT,
		B_MEDIA_LITTLE_ENDIAN,
		1470
	};
	
	sms_player = new SmsSoundPlayer(&format, "sms_plus");
	if(sms_player->InitCheck() != B_OK) {
		delete sms_player; sms_player = 0;
		return false;
	}
	
	sms_player->InitAll(2940);
	return true;
}

void SmsWindow::update_audio()
{
	static const int len = 2940;
	short *buf = (short *)malloc(len);

	uint i, j = 0;
	
	for(i = 0; i < len >> 2; i++) {
		buf[j] = (snd.buffer[0][i]);
		j++;
		buf[j] = (snd.buffer[1][i]);
		j++;
	}
	
	sms_player->WriteBufferSync(buf, len);
	free(buf);
}

bool SmsWindow::OpenJoystick(int index, Joystick *stick, bool value)
{
	char devName[B_OS_NAME_LENGTH];
	BString name;
	if(stick->GetDeviceName(index, devName) != B_OK) {
		return false;
	}
	if(stick->Open(devName, true) < B_OK) {
		return false;
	}
	if (stick->GetControllerName(&name) != B_OK) {
		return false;
	}
	stick->SwapVerticalAxes(value);
	return true;
}

void SmsWindow::input_init()
{
	joystick0 = new Joystick();
	joystick1 = new Joystick();
	
	if(!OpenJoystick(0, joystick0, settings.joypad0.swapy)) {
		delete joystick0; joystick0 = 0;
	}
	
	if(!OpenJoystick(1, joystick1, settings.joypad1.swapy)) {
		delete joystick1; joystick1 = 0;
	}
	sms_input_id = spawn_thread(sms_input_thread, "sms_input_thread", B_NORMAL_PRIORITY, this);
	resume_thread(sms_input_id);
}

void SmsWindow::ui_init()
{
	CheckMenuItem(ID_OPTIONS_SPECIAL_MODE0, settings.uisettings.mode == 0 ? true : false);
	CheckMenuItem(ID_OPTIONS_SPECIAL_MODE1, settings.uisettings.mode == 1 ? true : false);
	CheckMenuItem(ID_OPTIONS_SPECIAL_MODE2, settings.uisettings.mode == 2 ? true : false);
	CheckMenuItem(ID_OPTIONS_SPECIAL_MODE3, settings.uisettings.mode == 3 ? true : false);
	CheckMenuItem(ID_OPTIONS_SPECIAL_MODE4, settings.uisettings.mode == 4 ? true : false);
	CheckMenuItem(ID_OPTIONS_SPECIAL_MODE5, settings.uisettings.mode == 5 ? true : false);
	CheckMenuItem(ID_OPTIONS_SPECIAL_MODE6, settings.uisettings.mode == 6 ? true : false);
	if(settings.uisettings.mute) CheckMenuItem(ID_OPTIONS_MUTE, true);
	
	if(!sound_init()) {
		EnableMenuItem(ID_OPTIONS_MUTE, false);
		CheckMenuItem(ID_OPTIONS_MUTE, true);
		settings.uisettings.mute = true;
	}
}

#include "font1.h"

void SmsWindow::putpixel(int x, int y, uint32 col)
{
/*
	uint16 *dst = (uint16 *)sms_bmp->Bits() + ((y) * sms_bmp->BytesPerRow() >> 1) + x;
	dst[0] = (((col & 0xf8) >> 3) | ((col & 0xfc00) >> 5) | ((col & 0xf8) >> 8));
*/
	x += m_bounds.left;
	y += m_bounds.top;
	for(uint32 i = 0; i < m_numcliprects; i++)
		if(IsFullScreen() || (x >= m_cliplist[i].left && x <= m_cliplist[i].right) && (y >= m_cliplist[i].top && y <= m_cliplist[i].bottom)) {
		uint8 *dst = (uint8 *)(m_bits + ((m_rowbytes) * (y-m_bounds.top)));
		switch(m_format) {
			case B_RGB32:
				*((uint32 *)(dst) + x) = col;
			break;
			case B_RGB16:
				*((uint16 *)(dst) + x) = (((col & 0xf8) >> 3) | ((col & 0xfc00) >> 5) | ((col & 0xf8) >> 8));
			break;
			case B_CMAP8:
				*((uint8 *)(dst) + x) = (col >> 8) & 0xff;
			break;
			default:
			break;
		}
	}
}

void SmsWindow::DrawMessage(const unsigned char *message, int left, int top)
{
	int bitpos = 0;
	m_locker->Lock();
	if(m_connected) {
		int len = strlen((char *)message);
		for(int i = 0; i < len; i++, left += 8) {
			uint8 *data = &(fontdata_8x16[message[i] * 16]);
		for(int y = 0; y < 16; y++, bitpos = 7)
			for(int x = 0; x < 8; x++, bitpos--)
				(data[y] & (1 << bitpos)) ? putpixel(x + left, y + top, 0xff00ff00) : putpixel(x + left, y + top, 0);
		}
	}
	m_locker->Unlock();
}

SmsWindow::~SmsWindow()
{
	sms_quitting = true;
	snooze(32000);
	kill_thread(sms_input_id);
	int32 res;
	wait_for_thread(sms_id, &res);
	delete_sem(sms_lock_sem);
	delete sms_bmp;
	delete sms_back_bmp;
	delete sms_filepanel;
	delete m_locker;
	if(joystick0) {
		joystick0->Close();
		delete joystick0;
	}
	if(joystick1) {
		joystick1->Close();
		delete joystick1;
	}
	SetSettings(settings);
	SmsInterface::save_sram();
    system_shutdown();
    if(sms_player) {
    	sms_player->Stop();
    	delete sms_player;
    }
}
