#ifndef __SMSWINDOW_H__
#define __SMSWINDOW_H__

#include <Application.h>
#include <DirectWindow.h>
#include <Bitmap.h>
#include <View.h>
#include <FilePanel.h>
#include <Path.h>
#include <Locker.h>
#include <Screen.h>
#include <Alert.h>
#include <PushGameSound.h>
#include <stdio.h>
#include <MediaDefs.h>
#include "smssettings.h"

class SmsKeyInputWindow;
class SmsJoyInputWindow;
class GameMenuBar;
class SmsSoundPlayer;
class Joystick;
class SmsRefreshWindow;

class SmsWindow : public BDirectWindow
{
public:
	SmsWindow(BRect);
	virtual ~SmsWindow();
	virtual bool QuitRequested() { be_app->PostMessage(B_QUIT_REQUESTED); return(true); }
	virtual void MessageReceived(BMessage *m);
	virtual void DirectConnected(direct_buffer_info *info);
	virtual void WindowActivated(bool active);
	virtual void ResizeTo(float width, float height);
	static int32 sms_thread(void *o);
	int32 sms_thread();
	virtual void sms_emulate();
	virtual void update_video();
	virtual void update_videofull();
	virtual bool LoadAndRun(const entry_ref &ref);
	virtual void ClearScreen();
	virtual void init_machine();
	static int32 sms_drawing_thread(void *o);
	int32 sms_drawing_thread();
	static int32 sms_input_thread(void *o);
	int32 sms_input_thread();
	void update_input();
	virtual bool sound_init();
	virtual void input_init();
	virtual void ui_init();
	virtual void update_audio();
	virtual void InitMenuBar();
	virtual void SetGraphicsMode(int g_mode);
	virtual void DirectClear();
	virtual void CheckMenuItem(const uint32 msg, bool check);
	virtual void EnableMenuItem(const uint32 msg, bool enable);
	virtual void DoAlert();
	virtual bool OpenJoystick(int index, Joystick *stick, bool value);
	virtual void DrawMessage(const unsigned char *message, int left, int top);
	virtual void putpixel(int x, int y, uint32 col);
	virtual void StartTime(bigtime_t &start);
private:
	SmsKeyInputWindow *sms_key0_input_window;
	SmsKeyInputWindow *sms_key1_input_window;
	SmsJoyInputWindow *sms_joy0_input_window;
	SmsJoyInputWindow *sms_joy1_input_window;
	SmsRefreshWindow *sms_refresh_window;
	BBitmap *sms_bmp, *sms_back_bmp;
	BView *sms_view;
	uint8 key[256];
	BFilePanel *sms_filepanel;
	thread_id sms_id;
	bool sms_running, sms_quitting;
	volatile int frame_count, frames_rendered, frame_skip, skip;
	sem_id sms_lock_sem;
	thread_id sms_draw_id;
	thread_id sms_input_id;
	int32 _menubar_height;
	GameMenuBar *g_menu;
	
	// DirectWindow variables
	BLocker *m_locker;
	int32 m_bitsperpixel;
	int32 m_rowbytes;
	color_space m_format;
	uint32 m_numcliprects;
	clipping_rect *m_cliplist;
	clipping_rect m_bounds;
	uint8 *m_bits;
	bool m_connected;
	bool needs_clear;
	
	Joystick *joystick0, *joystick1;
	smssettings settings;
	display_mode prev_mode, new_mode, cur_mode;
	bool mode_is_valid;
	bigtime_t start;
	SmsSoundPlayer *sms_player;
};

#endif
